 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Java Time Zones
 *    Copyright (c) 2011 Marcin Gutman
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package com.example;

import com.sun.star.uno.XComponentContext;
import com.sun.star.lib.uno.helper.Factory;
import com.sun.star.lang.XSingleComponentFactory;
import com.sun.star.registry.XRegistryKey;
import com.sun.star.lib.uno.helper.WeakBase;
import name.mgplldz.javatimezones.office.JavaTimeZonesOffice;

public final class JavaTimeZonesAddInImpl extends WeakBase
        implements com.example.XJavaTimeZonesAddIn,
        com.sun.star.lang.XServiceInfo,
        com.sun.star.lang.XLocalizable {

    private final XComponentContext m_xContext;
    private static final String m_implementationName = JavaTimeZonesAddInImpl.class.getName();
    private static final String[] m_serviceNames = {
        "com.example.JavaTimeZonesAddIn"};
    private com.sun.star.lang.Locale m_locale = new com.sun.star.lang.Locale();

    public JavaTimeZonesAddInImpl(XComponentContext context) {
        m_xContext = context;
    }

    public static XSingleComponentFactory __getComponentFactory(String sImplementationName) {
        XSingleComponentFactory xFactory = null;

        if (sImplementationName.equals(m_implementationName)) {
            xFactory = Factory.createComponentFactory(JavaTimeZonesAddInImpl.class, m_serviceNames);
        }
        return xFactory;
    }

    public static boolean __writeRegistryServiceInfo(XRegistryKey xRegistryKey) {
        return Factory.writeRegistryServiceInfo(m_implementationName,
                m_serviceNames,
                xRegistryKey);
    }

    // com.example.XJavaTimeZonesAddIn:
    /*
     * JTZDate
     */
    public String jtz_date_reformat(String dateStr, String formatStrOld,
            String formatStrNew, Object tzIDObj, Object locIdObj) {
        return JavaTimeZonesOffice.dateReformat(dateStr, formatStrOld,
                formatStrNew, tzIDObj, locIdObj);
    }

    public String jtz_date_add(String dateStr, String formatStr, int field,
            int value, Object tzId, Object locId) {
        return JavaTimeZonesOffice.dateAdd(dateStr, formatStr, field, value,
                tzId, locId);
    }

    public String jtz_date_tz(String dateStr, String formatStr,
            int longName, Object locIdObj) {
        return JavaTimeZonesOffice.dateTz(dateStr, formatStr, longName,
                locIdObj);
    }

    public int jtz_date_tz_offset(String dateStr, String formatStr, int raw) {
        return JavaTimeZonesOffice.dateTzOffset(dateStr, formatStr, raw);
    }

    public String jtz_date_is_dst(String dateStr, String formatStr,
            Object tzIDObj) {
        return JavaTimeZonesOffice.dateIsDST(dateStr, formatStr, tzIDObj);
    }

    public double jtz_date_parse(String dateStr, String formatStr,
            Object tzIDObj) {
        return JavaTimeZonesOffice.dateParse(dateStr, formatStr, tzIDObj);
    }

    public double[][] jtz_date_split(String dateString, String formatString,
            Object tzIDObj, Object[] fieldsObj) {
        return JavaTimeZonesOffice.dateSplit(dateString, formatString, tzIDObj,
                fieldsObj);
    }

    /*
     * JTZUnix
     */
    public double jtz_unix_add(double millisDbl, String tzID, int field,
            int value) {
        return JavaTimeZonesOffice.unixAdd(millisDbl, tzID, field, value);
    }

    public double jtz_unix_now() {
        return JavaTimeZonesOffice.unixNow();
    }

    public String jtz_unix_format(double millisDbl, String formatStr,
            String tzID, Object locIDObj) {
        return JavaTimeZonesOffice.unixFormat(millisDbl, formatStr, tzID,
                locIDObj);
    }

    public String jtz_unix_is_dst(double millisDbl, String tzID) {
        return JavaTimeZonesOffice.unixIsDST(millisDbl, tzID);
    }

    public double[][] jtz_unix_split(double millisDbl, String tzID,
            Object[] fieldsObj) {
        return JavaTimeZonesOffice.unixSplit(millisDbl, tzID, fieldsObj);
    }

    public int jtz_unix_tz_offset(double millisDbl, String tzID,
            int offsetType) {
        return JavaTimeZonesOffice.unixTzOffset(millisDbl, tzID, offsetType);
    }

    public String jtz_unix_timer(double millisDbl) {
        return JavaTimeZonesOffice.unixTimer(millisDbl);
    }

    public int[][] jtz_unix_timer_split(double millisDbl) {
        return JavaTimeZonesOffice.unixTimerSplit(millisDbl);
    }

    /*
     * JTZHelp
     */
    public int jtz_f_year() {
        return JavaTimeZonesOffice.fYear();
    }

    public int jtz_f_month() {
        return JavaTimeZonesOffice.fMonth();
    }

    public int jtz_f_day() {
        return JavaTimeZonesOffice.fDay();
    }

    public int jtz_f_hour() {
        return JavaTimeZonesOffice.fHour();
    }

    public int jtz_f_minute() {
        return JavaTimeZonesOffice.fMinute();
    }

    public int jtz_f_second() {
        return JavaTimeZonesOffice.fSecond();
    }

    public int jtz_f_millis() {
        return JavaTimeZonesOffice.fMillis();
    }

    public int jtz_f_raw_offset() {
        return JavaTimeZonesOffice.fRawOffset();
    }

    public int jtz_f_dst_offset() {
        return JavaTimeZonesOffice.fDstOffset();
    }

    public int jtz_f_total_offset() {
        return JavaTimeZonesOffice.fTotalOffset();
    }

    public int jtz_mls_hour() {
        return JavaTimeZonesOffice.hourMls();
    }

    public int jtz_mls_minute() {
        return JavaTimeZonesOffice.minuteMls();
    }

    public int jtz_mls_second() {
        return JavaTimeZonesOffice.secondMls();
    }

    public String[][] jtz_tz_ids(Object allLangsObj, Object locIDObj,
            Object[] searchStrObj) {
        return JavaTimeZonesOffice.tzIDs(allLangsObj, locIDObj, searchStrObj);
    }

    public String[][] jtz_tz_ids_by_raw_offset(int millis, Object locIDObj) {
        return JavaTimeZonesOffice.tzByRawOffset(millis, locIDObj);
    }

    public String[][] jtz_loc_ids(Object allLangsObj, Object locIDObj,
            Object[] searchStrObj) {
        return JavaTimeZonesOffice.locIDs(allLangsObj, locIDObj, searchStrObj);
    }

    public String[][] jtz_tz_loc(String search, Object locIDObj) {
        return JavaTimeZonesOffice.getTzLoc(search, locIDObj);
    }

    public Object[][] jtz_tz_info(String tzID, Object namesObj, Object longNameObj,
            Object locIDObj) {
        return JavaTimeZonesOffice.tzInfo(tzID, namesObj, longNameObj, locIDObj);
    }

    public String[][] jtz_format_codes() {
        return JavaTimeZonesOffice.getFormatCodes();
    }

    // com.sun.star.lang.XServiceInfo:
    public String getImplementationName() {
        return m_implementationName;
    }

    public boolean supportsService(String sService) {
        int len = m_serviceNames.length;

        for (int i = 0; i < len; i++) {
            if (sService.equals(m_serviceNames[i])) {
                return true;
            }
        }
        return false;
    }

    public String[] getSupportedServiceNames() {
        return m_serviceNames;
    }

    // com.sun.star.lang.XLocalizable:
    public void setLocale(com.sun.star.lang.Locale eLocale) {
        m_locale = eLocale;
    }

    public com.sun.star.lang.Locale getLocale() {
        return m_locale;
    }
}
