 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Java Time Zones
 *    Copyright (c) 2011 Marcin Gutman
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.javatimezones.office;

import com.sun.star.lang.IllegalArgumentException;
import com.sun.star.uno.AnyConverter;
import com.sun.star.uno.Type;
import java.util.logging.Level;
import java.util.logging.Logger;
import name.mgplldz.javatimezones.JTZDate;
import name.mgplldz.javatimezones.JTZExceptions.JTZExtractTzException;
import name.mgplldz.javatimezones.JTZExceptions.JTZFieldException;
import name.mgplldz.javatimezones.JTZExceptions.JTZFormatException;
import name.mgplldz.javatimezones.JTZExceptions.JTZLocaleException;
import name.mgplldz.javatimezones.JTZExceptions.JTZTimeZoneException;
import name.mgplldz.javatimezones.JTZHelp;
import name.mgplldz.javatimezones.JTZUnix;

public class JavaTimeZonesOffice {

    /*
     * JTZDate
     */
    public static String dateReformat(String dateStr, String formatStrOld,
            String formatStrNew, Object tzIDObj, Object locIdObj) {
        String out = null;
        String tzId = "";
        String locId = "";
        try {
            if (!AnyConverter.isVoid(tzIDObj)) {
                tzId = AnyConverter.toString(tzIDObj);
            }
            if (!AnyConverter.isVoid(locIdObj)) {
                locId = AnyConverter.toString(locIdObj);
            }
            out = JTZDate.dateReformat(dateStr, formatStrOld, formatStrNew,
                    tzId, locId);
        } catch (JTZTimeZoneException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        } catch (JTZLocaleException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        } catch (JTZFormatException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IllegalArgumentException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        }
        return out;
    }

    public static String dateAdd(String dateStr, String formatStr,
            int field, int value, Object tzIdObj, Object locIdObj) {
        String out = null;
        String tzId = "";
        String locId = "";
        try {
            if (!AnyConverter.isVoid(tzIdObj)) {
                tzId = AnyConverter.toString(tzIdObj);
            }
            if (!AnyConverter.isVoid(locIdObj)) {
                locId = AnyConverter.toString(locIdObj);
            }
            out = JTZDate.dateAdd(dateStr, formatStr, field, value, tzId,
                    locId);
        } catch (JTZTimeZoneException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        } catch (JTZLocaleException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        } catch (JTZFormatException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        } catch (JTZExtractTzException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IllegalArgumentException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        }
        return out;
    }

    public static String dateTz(String dateStr, String formatStr,
            int longName, Object locIdObj) {
        String out = "";
        String locId = "";
        try {
            if (!AnyConverter.isVoid(locIdObj)) {
                locId = AnyConverter.toString(locIdObj);
            }
            out = JTZDate.dateTz(dateStr, formatStr, longName, locId);
        } catch (JTZTimeZoneException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        } catch (JTZLocaleException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        } catch (JTZFormatException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IllegalArgumentException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        }
        return out;
    }

    public static int dateTzOffset(String dateStr, String formatStrWith,
            int raw) {
        int out = 0;
        try {
            out = JTZDate.dateTzOffset(dateStr, formatStrWith, raw);
        } catch (JTZTimeZoneException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        } catch (JTZLocaleException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        } catch (JTZFormatException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        } catch (JTZFieldException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        }
        return out;
    }

    public static String dateIsDST(String dateStr, String formatStr,
            Object tzIDObj) {
        boolean out = false;
        String tzID = "";
        try {
            if (!AnyConverter.isVoid(tzIDObj)) {
                tzID = AnyConverter.toString(tzIDObj);
            }
            out = JTZDate.dateIsDST(dateStr, formatStr, tzID);
        } catch (JTZTimeZoneException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        } catch (JTZLocaleException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        } catch (JTZFormatException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        } catch (JTZExtractTzException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IllegalArgumentException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        }
        if (!out) {
            return "false";
        } else {
            return "true";
        }
    }

    public static double dateParse(String dateStr, String formatStr, Object tzIDObj) {
        String tzID = "";
        double out = 0;
        try {
            if (!AnyConverter.isVoid(tzIDObj)) {
                tzID = AnyConverter.toString(tzIDObj);
            }
            out = JTZDate.dateParse(dateStr, formatStr, tzID);
        } catch (JTZTimeZoneException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        } catch (JTZLocaleException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        } catch (JTZFormatException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IllegalArgumentException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        }
        return out;
    }

    public static double[][] dateSplit(String dateString, String formatString,
            Object tzIDObj, Object[] fieldsObj) {
        long out[] = null;
        double outFinal[][] = null;
        String tzID = "";
        int fields[] = null;
        try {
            if (!AnyConverter.isVoid(tzIDObj)) {
                tzID = AnyConverter.toString(tzIDObj);
            }
            if (!AnyConverter.isVoid(fieldsObj)) {
                fields = new int[fieldsObj.length];
                int i = 0;
                for (Object o : fieldsObj) {
                    fields[i] = (int) Math.round(AnyConverter.toDouble(o));
                    ++i;
                }
            }
            out = JTZDate.dateSplit(dateString, formatString, tzID, fields);
            outFinal = new double[out.length][1];
            int i = 0;
            for (long l : out) {
                outFinal[i][0] = l;
                ++i;
            }
        } catch (JTZTimeZoneException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        } catch (JTZLocaleException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        } catch (JTZFormatException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        } catch (JTZFieldException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IllegalArgumentException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        }
        return outFinal;
    }

    /*
     * JTZUnix
     */
    public static double unixAdd(double millisDbl, String tzID, int field, int value) {
        double out = 0.0;
        long millis = Math.round(millisDbl);
        try {
            out = JTZUnix.unixAdd(millis, tzID, field, value);
        } catch (JTZTimeZoneException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        }
        return out;
    }

    public static double unixNow() {
        return JTZUnix.unixNow();
    }

    public static String unixFormat(double millisDbl, String formatStr, String tzID,
            Object locIDObj) {
        String out = "";
        String locID = "";
        long millis = Math.round(millisDbl);
        try {
            if (!AnyConverter.isVoid(locIDObj)) {
                locID = AnyConverter.toString(locIDObj);

            }
            out = JTZUnix.unixFormat(millis, formatStr, tzID, locID);
        } catch (JTZTimeZoneException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        } catch (JTZLocaleException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        } catch (JTZFormatException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IllegalArgumentException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        }
        return out;
    }

    public static String unixIsDST(double millisDbl, String tzID) {
        boolean out = false;
        long millis = Math.round(millisDbl);
        try {
            out = JTZUnix.unixIsDST(millis, tzID);
        } catch (JTZTimeZoneException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        }
        if (out) {
            return "true";
        } else {
            return "false";
        }
    }

    public static double[][] unixSplit(double millisDbl, String tzID,
            Object[] fieldsObj) {
        long out[] = null;
        double outFinal[][] = null;
        int fields[] = null;
        long millis = Math.round(millisDbl);
        try {
            if (!AnyConverter.isVoid(fieldsObj)) {
                fields = new int[fieldsObj.length];
                int i = 0;
                for (Object o : fieldsObj) {
                    fields[i] = (int) Math.round(AnyConverter.toDouble(o));
                    ++i;
                }
            }
            out = JTZUnix.unixSplit(millis, tzID, fields);
            outFinal = new double[out.length][1];
            int i = 0;
            for (long l : out) {
                outFinal[i][0] = l;
                ++i;
            }
        } catch (JTZTimeZoneException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        } catch (JTZFieldException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IllegalArgumentException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        }
        return outFinal;
    }

    public static int unixTzOffset(double millisDbl, String tzID, int offsetType) {
        int out = 0;
        long millis = Math.round(millisDbl);
        try {
            out = JTZUnix.unixTzOffset(millis, tzID, offsetType);
        } catch (JTZTimeZoneException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        } catch (JTZFieldException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        }
        return out;
    }

    public static String unixTimer(double millisDbl) {
        long millis = Math.round(millisDbl);
        return JTZUnix.timer(millis);
    }

    public static int[][] unixTimerSplit(double millisDbl) {
        Integer out[] = JTZUnix.timerSplit(Math.round(millisDbl));
        int outFinal[][] = new int[out.length][1];
        int i = 0;
        for (Integer integ : out) {
            outFinal[i][0] = integ.intValue();
            ++i;
        }
        return outFinal;
    }
    /*
     * JTZHelp
     */

    public static int fYear() {
        return JTZHelp.fYear();
    }

    public static int fMonth() {
        return JTZHelp.fMonth();
    }

    public static int fDay() {
        return JTZHelp.fDay();
    }

    public static int fHour() {
        return JTZHelp.fHour();
    }

    public static int fMinute() {
        return JTZHelp.fMinute();
    }

    public static int fSecond() {
        return JTZHelp.fSecond();
    }

    public static int fMillis() {
        return JTZHelp.fMillis();
    }

    public static int fRawOffset() {
        return JTZHelp.fRawOffset();
    }

    public static int fDstOffset() {
        return JTZHelp.fDstOffset();
    }

    public static int fTotalOffset() {
        return JTZHelp.fTotalOffset();
    }

    public static int hourMls() {
        return JTZHelp.hourMls();
    }

    public static int minuteMls() {
        return JTZHelp.minMls();
    }

    public static int secondMls() {
        return JTZHelp.secMls();
    }

    public static String[][] tzIDs(Object allLangsObj, Object locIDObj,
            Object[] searchStrObj) {
        int allLangs = 0;
        String locID = "";
        String[] searchStr = new String[0];
        String out[][] = new String[0][0];
        try {
            if (!AnyConverter.isVoid(allLangsObj)) {
                allLangs = (int) Math.round(AnyConverter.toDouble(allLangsObj));
            }
            if (!AnyConverter.isVoid(locIDObj)) {
                locID = AnyConverter.toString(locIDObj);
            }
            if (!AnyConverter.isVoid(searchStrObj)) {
                searchStr = new String[searchStrObj.length];
                int i = 0;
                for (Object o : searchStrObj) {
                    searchStr[i] = AnyConverter.toObject(Type.STRING, o).toString();
                    ++i;
                }
            }
            out = JTZHelp.getTzIDs(allLangs, locID, searchStr);
        } catch (JTZLocaleException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
            throw new com.sun.star.uno.RuntimeException();
        } catch (IllegalArgumentException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        }
        return out;
    }

    public static String[][] locIDs(Object allLangsObj, Object locIDObj,
            Object... searchStrObj) {
        int allLangs = 0;
        String locID = "";
        String[] searchStr = new String[0];
        String out[][] = new String[0][0];
        try {
            if (!AnyConverter.isVoid(allLangsObj)) {
                allLangs = (int) Math.round(AnyConverter.toDouble(allLangsObj));
            }
            if (!AnyConverter.isVoid(locIDObj)) {
                locIDObj = AnyConverter.toString(locIDObj);
            }
            if (!AnyConverter.isVoid(searchStrObj)) {
                searchStr = new String[searchStrObj.length];
                int i = 0;
                for (Object o : searchStrObj) {
                    searchStr[i] = AnyConverter.toObject(Type.STRING, o).toString();
                    ++i;
                }
            }
            out = JTZHelp.getLocIDs(allLangs, locID, searchStr);
        } catch (JTZLocaleException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
            throw new com.sun.star.uno.RuntimeException();
        } catch (IllegalArgumentException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        }
        return out;
    }

    public static String[][] getTzLoc(String search, Object locIDObj) {
        String locID = "";
        String out[][] = null;
        try {
            if (!AnyConverter.isVoid(locIDObj)) {
                locID = AnyConverter.toString(locIDObj);
            }
            return JTZHelp.getTzLoc(search, locID);
        } catch (JTZLocaleException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).
                    log(Level.SEVERE, null, ex);
        } catch (IllegalArgumentException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        }
        return out;
    }

    public static Object[][] tzInfo(String tzID, Object namesObj, Object longNameObj,
            Object locIDObj) {
        Object out[] = null;
        int names = 0;
        int longName = 0;
        String locID = "";
        Object outFinal[][] = null;
        try {
            if (!AnyConverter.isVoid(namesObj)) {
                names = (int) Math.round(AnyConverter.toDouble(namesObj));
            }
            if (!AnyConverter.isVoid(longNameObj)) {
                longName = (int) Math.round(AnyConverter.toDouble(longNameObj));
            }
            if (!AnyConverter.isVoid(locIDObj)) {
                locID = AnyConverter.toString(locIDObj);
            }
            out = JTZHelp.tzInfo(tzID, names, longName, locID);
            outFinal = new Object[out.length][1];
            int i = 0;
            for (Object o : out) {
                outFinal[i][0] = o;
                ++i;
            }
        } catch (JTZTimeZoneException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        } catch (JTZLocaleException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IllegalArgumentException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        }
        return outFinal;
    }

    public static String[][] tzByRawOffset(int millis, Object locIDObj) {
        String locID = "";
        String out[][] = null;
        try {
            if (!AnyConverter.isVoid(locIDObj)) {
                locID = AnyConverter.toString(locIDObj);
            }
            out = JTZHelp.getTzIDsByRawOffset(millis, locID);
        } catch (JTZLocaleException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IllegalArgumentException ex) {
            Logger.getLogger(JavaTimeZonesOffice.class.getName()).log(Level.SEVERE, null, ex);
        }
        return out;
    }

    public static String[][] getFormatCodes() {
        return JTZHelp.getFormatCodes();
    }
}
