 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Java Time Zones
 *    Copyright (c) 2011 Marcin Gutman
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.javatimezones;

import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Locale;
import java.util.TimeZone;
import name.mgplldz.javatimezones.JTZExceptions.JTZExtractTzException;
import name.mgplldz.javatimezones.JTZExceptions.JTZFieldException;
import name.mgplldz.javatimezones.JTZExceptions.JTZFormatException;
import name.mgplldz.javatimezones.JTZExceptions.JTZLocaleException;
import name.mgplldz.javatimezones.JTZExceptions.JTZTimeZoneException;

public class JTZDate {

    public static String dateReformat(String dateStr, String formatStr,
            String formatStrNew, String tzID, String localeID)
            throws
            JTZFormatException,
            JTZTimeZoneException,
            JTZLocaleException {
        JTZDateFormat dfMy = new JTZDateFormat(dateStr, formatStr, tzID, localeID);
        long millis = dateParse(dateStr, formatStr, tzID);
        if (localeID != null) {
            if (localeID.isEmpty()) {
                localeID = dfMy.getLocParse().toString();
            }
        }
        dfMy = new JTZDateFormat(millis, formatStrNew, dfMy.getTz(), localeID);
        return dfMy.getDateStr();
    }

    public static String dateAdd(String dateStr, String formatStr, int field,
            int value, String timeZoneID, String localeID)
            throws
            JTZTimeZoneException,
            JTZLocaleException,
            JTZFormatException,
            JTZExtractTzException {
        JTZDateFormat dfMy = new JTZDateFormat(dateStr, formatStr, timeZoneID, localeID);
        long millis = JTZUnix.unixAdd(dfMy.getMillis(), dfMy.getTz(), field, value);
        if (localeID != null) {
            if (localeID.isEmpty()) {
                localeID = dfMy.getLocParse().toString();
            }
        }
        dfMy = new JTZDateFormat(millis, formatStr, dfMy.getTz(), localeID);
        return dfMy.getDateStr();
    }

    public static String dateTz(String dateStr, String formatStrWith,
            int longName, String locId)
            throws
            JTZTimeZoneException,
            JTZLocaleException,
            JTZFormatException {
        String tzName;
        JTZDateFormat dfMy = new JTZDateFormat(dateStr, formatStrWith,
                JTZHelp.GMT_ID, locId);
        if (longName == 1) {
            tzName = dfMy.getTz().getDisplayName(dfMy.getTz().inDaylightTime(
                    new Date(dfMy.getMillis())), TimeZone.LONG, dfMy.getLocOut());
        } else {
            tzName = dfMy.getTz().getDisplayName(dfMy.getTz().inDaylightTime(
                    new Date(dfMy.getMillis())), TimeZone.SHORT, dfMy.getLocOut());
        }
        return tzName;
    }

    public static int dateTzOffset(String dateStr, String formatStr,
            int offsetType)
            throws
            JTZTimeZoneException,
            JTZLocaleException,
            JTZFormatException,
            JTZFieldException {
        JTZDateFormat dfMy = new JTZDateFormat(dateStr, formatStr,
                JTZHelp.GMT_ID, Locale.getDefault().toString());
        int offset = 0;
        switch (offsetType) {
            case GregorianCalendar.ZONE_OFFSET:
                offset = dfMy.getTz().getRawOffset();
                break;
            case GregorianCalendar.DST_OFFSET:
                offset = dfMy.getTz().getOffset(dfMy.getMillis());
                break;
            case JTZHelp.TOTAL_OFFSET:
                offset = dfMy.getTz().getRawOffset()
                        + dfMy.getTz().getOffset(dfMy.getMillis());
                break;
            default:
                throw new JTZFieldException("" + offsetType);
        }
        return offset;
    }

    public static boolean dateIsDST(String dateStr, String formatStr,
            String tzID)
            throws
            JTZTimeZoneException,
            JTZLocaleException,
            JTZFormatException,
            JTZExtractTzException {
        JTZDateFormat dfMy = new JTZDateFormat(dateStr, formatStr, tzID,
                Locale.getDefault().toString());
        return dfMy.getTz().inDaylightTime(new Date(dfMy.getMillis()));
    }

    public static Long dateParse(String dateStr, String formatStr,
            String tzID)
            throws
            JTZTimeZoneException,
            JTZLocaleException,
            JTZFormatException {
        JTZDateFormat dfMy = new JTZDateFormat(dateStr, formatStr, tzID,
                Locale.getDefault().toString());
        return dfMy.getMillis();
    }

    public static long[] dateSplit(String dateStr, String formatStr,
            String tzID, int... fields) throws
            JTZTimeZoneException,
            JTZLocaleException,
            JTZFormatException,
            JTZFieldException {
        JTZDateFormat dfMy = new JTZDateFormat(dateStr, formatStr, tzID,
                Locale.getDefault().toString());
        return JTZUnix.unixSplit(dfMy.getMillis(), dfMy.getTz(), fields);
    }
}
