 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Java Time Zones
 *    Copyright (c) 2011 Marcin Gutman
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.javatimezones;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Locale;
import java.util.TimeZone;
import name.mgplldz.javatimezones.JTZExceptions.JTZFormatException;
import name.mgplldz.javatimezones.JTZExceptions.JTZLocaleException;
import name.mgplldz.javatimezones.JTZExceptions.JTZTimeZoneException;

class JTZDateFormat {

    private SimpleDateFormat sdfMy;
    private Locale locParseMy;
    private Locale locOutputMy;
    private Long millisOutput;
    private String dateStrOutput;

    JTZDateFormat(String dateStr, String formatStr, String tzID,
            String locOutID)
            throws
            JTZFormatException,
            JTZLocaleException,
            JTZTimeZoneException {
        TimeZone tz = JTZHelp.getTzByID(tzID);
        Locale locOut = JTZHelp.getLocByID(locOutID);
        setSDFDate(dateStr, formatStr, tz, locOut);
    }

    JTZDateFormat(Long millis, String formatStr, String tzID, String locOutID)
            throws
            JTZFormatException,
            JTZLocaleException,
            JTZTimeZoneException {
        TimeZone tz = JTZHelp.getTzByID(tzID);
        Locale locOut = JTZHelp.getLocByID(locOutID);
        setSDFMillis(millis, formatStr, tz, locOut);
    }

    JTZDateFormat(Long millis, String formatStr, TimeZone tz, String locOutID)
            throws
            JTZFormatException,
            JTZLocaleException,
            JTZTimeZoneException {
        Locale locOut = JTZHelp.getLocByID(locOutID);
        setSDFMillis(millis, formatStr, tz, locOut);
    }

    TimeZone getTz() {
        return sdfMy.getTimeZone();
    }

    Locale getLocOut() {
        return locOutputMy;
    }

    Locale getLocParse() {
        return locParseMy;
    }

    Long getMillis() {
        return millisOutput;
    }

    String getDateStr() {
        return dateStrOutput;
    }

    private void setSDFDate(String dateStr, String formatStr, TimeZone tz,
            Locale locOutput)
            throws
            JTZFormatException,
            JTZLocaleException,
            JTZTimeZoneException {
        if (dateStr == null) {
            throw new JTZFormatException("Date string == null");
        } else if (dateStr.isEmpty()) {
            throw new JTZFormatException("Date string is empty.");
        }
        if (formatStr == null) {
            throw new JTZFormatException("Format string == null");
        } else if (formatStr.isEmpty()) {
            throw new JTZFormatException("Format string is empty.");
        }
        if (locOutput == null) {
            throw new JTZLocaleException("Locale == null");
        }
        if (tz == null) {
            throw new JTZTimeZoneException("TimeZone == null");
        }
        locOutputMy = locOutput;
        GregorianCalendar gc = new GregorianCalendar();
        try {
            sdfMy = new SimpleDateFormat(formatStr, Locale.getDefault());
            sdfMy.setCalendar(gc);
            sdfMy.setTimeZone(tz);
            millisOutput = sdfMy.parse(dateStr).getTime();
            locParseMy = Locale.getDefault();
        } catch (ParseException ex1) {
            try {
                sdfMy = new SimpleDateFormat(formatStr, locOutput);
                sdfMy.setCalendar(gc);
                sdfMy.setTimeZone(tz);
                millisOutput = sdfMy.parse(dateStr).getTime();
                locParseMy = locOutput;
            } catch (ParseException ex2) {
                locParseMy = null;
                for (Locale loc : Locale.getAvailableLocales()) {
                    sdfMy = new SimpleDateFormat(formatStr, loc);
                    sdfMy.setCalendar(gc);
                    sdfMy.setTimeZone(tz);
                    try {
                        millisOutput = sdfMy.parse(dateStr).getTime();
                        locParseMy = loc;
                        break;
                    } catch (ParseException ignore) {
                    }
                }
            }
        }
        if (locParseMy == null) {
            sdfMy = null;
            String msg = "Date: " + dateStr
                    + "Format: " + formatStr;
            throw new JTZFormatException(msg);
        }
    }

    private void setSDFMillis(Long millis, String formatStr, TimeZone tz,
            Locale locOutput)
            throws
            JTZFormatException,
            JTZLocaleException,
            JTZTimeZoneException {
        if (formatStr == null) {
            throw new JTZFormatException("Format string == null");
        } else if (formatStr.isEmpty()) {
            throw new JTZFormatException("Format string is empty.");
        }
        if (locOutput == null) {
            throw new JTZLocaleException("Locale == null");
        }
        if (tz == null) {
            throw new JTZTimeZoneException("TimeZone == null");
        }
        locOutputMy = locOutput;
        GregorianCalendar gc = new GregorianCalendar();
        sdfMy = new SimpleDateFormat(formatStr, locOutputMy);
        sdfMy.setCalendar(gc);
        sdfMy.setTimeZone(tz);
        dateStrOutput = sdfMy.format(new Date(millis));
    }
}
