 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Java Time Zones
 *    Copyright (c) 2011 Marcin Gutman
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.javatimezones;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.GregorianCalendar;
import java.util.Locale;
import java.util.Set;
import java.util.TimeZone;
import java.util.TreeSet;
import name.mgplldz.javatimezones.JTZExceptions.JTZLocaleException;
import name.mgplldz.javatimezones.JTZExceptions.JTZTimeZoneException;

public class JTZHelp {

    private static final int SEC = 1000;
    private static final int MIN = 1000 * 60;
    private static final int HOUR = 1000 * 60 * 60;
    static final String GMT_ID = "GMT";
    static final String DATE_FRMT_STR_DEFAULT = "yyyy-MM-dd HH:mm:ss.S z";
    static final int TOTAL_OFFSET = 1001;

    public static String[][] getLocIDs(int allLangs, String locID,
            String... searchStr)
            throws JTZLocaleException {
        Locale locOut = JTZHelp.getLocByID(locID);
        if (searchStr.length <= 0) {
            searchStr = new String[]{""};
        } else {
            for (int i = 0; i < searchStr.length; ++i) {
                if (searchStr[i] == null) {
                    searchStr[i] = "";
                }
            }
        }
        Set<Locale> locSet = new TreeSet<Locale>(new Comparator<Locale>() {

            public int compare(Locale loc1, Locale loc2) {
                return loc1.toString().compareTo(loc2.toString());
            }
        });
        Locale loc;
        boolean foundFl = true;
        for (Locale locSearch : Locale.getAvailableLocales()) {
            if (allLangs == 1) {
                loc = locSearch;
            } else {
                loc = locOut;
            }
            for (Locale locItem : Locale.getAvailableLocales()) {
                for (String srch : searchStr) {
                    if (!(locItem.toString().toLowerCase().
                            contains(srch.toLowerCase()))
                            && !(locItem.getDisplayLanguage(loc).toLowerCase().
                            contains(srch.toLowerCase()))
                            && !(locItem.getDisplayCountry(loc).toLowerCase().
                            contains(srch.toLowerCase()))
                            && !(locItem.getDisplayName(loc).toLowerCase().
                            contains(srch.toLowerCase()))) {
                        foundFl = false;
                        break;
                    }
                }
                if (foundFl) {
                    locSet.add(locItem);
                }
                foundFl = true;
            }
            if (allLangs != 1) {
                break;
            }
        }
        String outArr[][] = new String[locSet.size()][4];
        int i = 0;
        for (Locale locS : locSet) {
            outArr[i][0] = locS.toString();
            outArr[i][1] = locS.getDisplayLanguage(locOut);
            outArr[i][2] = locS.getDisplayCountry(locOut);
            outArr[i][3] = locS.getDisplayName(locOut);
            ++i;
        }
        locSet.clear();
        return outArr;
    }

    public static String[][] getTzLoc(String search, String locIDOutput)
            throws
            JTZLocaleException {
        if (search == null) {
            search = "";
        }
        TimeZone tz;
        Locale locOutput = JTZHelp.getLocByID(locIDOutput);
        Set<Locale> locSet = new TreeSet<Locale>(new Comparator<Locale>() {

            public int compare(Locale loc1, Locale loc2) {
                return loc1.toString().compareTo(loc2.toString());
            }
        });
        for (Locale loc : Locale.getAvailableLocales()) {
            for (String tzID : TimeZone.getAvailableIDs()) {
                tz = TimeZone.getTimeZone(tzID);
                if (!(tz.getDisplayName(false, TimeZone.SHORT, loc).toLowerCase().
                        contains(search.toLowerCase()))
                        && !(tz.getDisplayName(false, TimeZone.LONG, loc).toLowerCase().
                        contains(search.toLowerCase()))
                        && !(tz.getDisplayName(true, TimeZone.SHORT, loc).toLowerCase().
                        contains(search.toLowerCase()))
                        && !(tz.getDisplayName(true, TimeZone.LONG, loc).toLowerCase().
                        contains(search.toLowerCase()))) {
                    // ignore
                } else {
                    locSet.add(loc);
                }
            }
        }
        String out[][] = new String[locSet.size()][4];
        int i = 0;
        for (Locale loc : locSet) {
            out[i][0] = loc.toString();
            out[i][1] = loc.getDisplayLanguage(locOutput);
            out[i][2] = loc.getDisplayCountry(locOutput);
            out[i][3] = loc.getDisplayName(locOutput);
            ++i;
        }
        locSet.clear();
        return out;
    }

    public static int secMls() {
        return SEC;
    }

    public static int minMls() {
        return MIN;
    }

    public static int hourMls() {
        return HOUR;
    }

    public static int fMillis() {
        return GregorianCalendar.MILLISECOND;
    }

    public static int fSecond() {
        return GregorianCalendar.SECOND;
    }

    public static int fMinute() {
        return GregorianCalendar.MINUTE;
    }

    public static int fHour() {
        return GregorianCalendar.HOUR_OF_DAY;
    }

    public static int fDay() {
        return GregorianCalendar.DAY_OF_MONTH;
    }

    public static int fMonth() {
        return GregorianCalendar.MONTH;
    }

    public static int fYear() {
        return GregorianCalendar.YEAR;
    }

    public static int fRawOffset() {
        return GregorianCalendar.ZONE_OFFSET;
    }

    public static int fDstOffset() {
        return GregorianCalendar.DST_OFFSET;
    }

    public static int fTotalOffset() {
        return TOTAL_OFFSET;
    }

    public static String[][] getFormatCodes() {
        ArrayList<String[]> arrLst = new ArrayList<String[]>();
        arrLst.add(new String[]{"G", "Era designator"});
        arrLst.add(new String[]{"y", "Year"});
        arrLst.add(new String[]{"M", "Month in year"});
        arrLst.add(new String[]{"w", "Week in year"});
        arrLst.add(new String[]{"W", "Week in month"});
        arrLst.add(new String[]{"D", "Day in year"});
        arrLst.add(new String[]{"d", "Day in month"});
        arrLst.add(new String[]{"F", "Day of week in month"});
        arrLst.add(new String[]{"E", "Dday in week"});
        arrLst.add(new String[]{"a", "AM/PM marker"});
        arrLst.add(new String[]{"H", "Hour in day (0-23)"});
        arrLst.add(new String[]{"k", "Hour in day (1-24)"});
        arrLst.add(new String[]{"K", "Hour in AM/PM (0-11)"});
        arrLst.add(new String[]{"h", "Hour in AM/PM (1-12)"});
        arrLst.add(new String[]{"m", "Minute in hour"});
        arrLst.add(new String[]{"s", "Second in minute"});
        arrLst.add(new String[]{"S", "Millisecond"});
        arrLst.add(new String[]{"z", "Time zone (general)"});
        arrLst.add(new String[]{"Z", "Time zone (RFC 822)"});
        arrLst.add(new String[]{"'", "Escape for text"});
        arrLst.add(new String[]{"'", "Single quote"});
        String codes[][] = new String[arrLst.size()][2];
        String s[] = null;
        for (int i = 0; i < arrLst.size(); ++i) {
            s = (String[]) arrLst.get(i);
            for (int k = 0; k < 2; ++k) {
                codes[i][k] = s[k];
            }
        }
        return codes;
    }

    public static Object[] tzInfo(String tzID, int names, int longName,
            String locID)
            throws JTZTimeZoneException,
            JTZLocaleException {
        Object info[];
        ArrayList<Object> infoLst = new ArrayList<Object>();
        TimeZone tz = getTzByID(tzID);
        Locale loc = getLocByID(locID);
        if (names == 1) {
            if (longName == 1) {
                infoLst.add(tz.getDisplayName(false, TimeZone.LONG, loc));
                infoLst.add(tz.getDisplayName(true, TimeZone.LONG, loc));
            } else {
                infoLst.add(tz.getDisplayName(false, TimeZone.SHORT, loc));
                infoLst.add(tz.getDisplayName(true, TimeZone.SHORT, loc));
            }
        } else {
            infoLst.add(tz.getRawOffset());
            infoLst.add(tz.getDSTSavings());
        }
        info = new Object[infoLst.size()];
        infoLst.toArray(info);
        infoLst.clear();
        return info;
    }

    public static String[][] getTzIDsByRawOffset(int millis, String locID)
            throws
            JTZLocaleException {
        Locale loc = getLocByID(locID);
        TimeZone tz;
        String idArr[] = TimeZone.getAvailableIDs(millis);
        int rows = idArr.length;
        String outArr[][] = new String[rows][5];
        for (int i = 0; i < rows; ++i) {
            tz = TimeZone.getTimeZone(idArr[i]);
            outArr[i][0] = tz.getID();
            outArr[i][1] = tz.getDisplayName(false, TimeZone.SHORT, loc);
            outArr[i][2] = tz.getDisplayName(false, TimeZone.LONG, loc);
            outArr[i][3] = tz.getDisplayName(true, TimeZone.SHORT, loc);
            outArr[i][4] = tz.getDisplayName(true, TimeZone.LONG, loc);
        }
        return outArr;
    }

    public static String[][] getTzIDs(int allLangs, String locID,
            String... searchStr)
            throws
            JTZLocaleException {
        Locale locOut = JTZHelp.getLocByID(locID);
        if (searchStr.length <= 0) {
            searchStr = new String[]{""};
        } else {
            for (int i = 0; i < searchStr.length; ++i) {
                if (searchStr[i] == null) {
                    searchStr[i] = "";
                }
            }
        }
        String tzIDsArr[] = TimeZone.getAvailableIDs();
        Set<String> tzIDSet = new TreeSet<String>();
        TimeZone tz;
        Locale loc;
        boolean foundFl = true;
        for (Locale locSearch : Locale.getAvailableLocales()) {
            if (allLangs == 1) {
                loc = locSearch;
            } else {
                loc = locOut;
            }
            for (String s : tzIDsArr) {
                tz = TimeZone.getTimeZone(s);
                for (String srch : searchStr) {
                    if (!(tz.getID().toLowerCase().
                            contains(srch.toLowerCase()))
                            && !(tz.getDisplayName(false, TimeZone.SHORT, loc).
                            toLowerCase().contains(srch.toLowerCase()))
                            && !(tz.getDisplayName(false, TimeZone.LONG, loc).
                            toLowerCase().contains(srch.toLowerCase()))
                            && !(tz.getDisplayName(true, TimeZone.SHORT, loc).
                            toLowerCase().contains(srch.toLowerCase()))
                            && !(tz.getDisplayName(true, TimeZone.LONG, loc).
                            toLowerCase().contains(srch.toLowerCase()))) {
                        foundFl = false;
                        break;
                    }
                }
                if (foundFl) {
                    tzIDSet.add(s);
                }
                foundFl = true;
            }
            if (allLangs != 1) {
                break;
            }
        }
        String outArr[][] = new String[tzIDSet.size()][5];
        int i = 0;
        for (String s : tzIDSet) {
            tz = TimeZone.getTimeZone(s);
            outArr[i][0] = tz.getID();
            outArr[i][1] = tz.getDisplayName(false, TimeZone.SHORT, locOut);
            outArr[i][2] = tz.getDisplayName(false, TimeZone.LONG, locOut);
            outArr[i][3] = tz.getDisplayName(true, TimeZone.SHORT, locOut);
            outArr[i][4] = tz.getDisplayName(true, TimeZone.LONG, locOut);
            ++i;
        }
        tzIDSet.clear();
        return outArr;
    }

    /*
     * Package
     */
    static int monthNb(int month) {
        switch (month) {
            case GregorianCalendar.JANUARY:
                return 1;
            case GregorianCalendar.FEBRUARY:
                return 2;
            case GregorianCalendar.MARCH:
                return 3;
            case GregorianCalendar.APRIL:
                return 4;
            case GregorianCalendar.MAY:
                return 5;
            case GregorianCalendar.JUNE:
                return 6;
            case GregorianCalendar.JULY:
                return 7;
            case GregorianCalendar.AUGUST:
                return 8;
            case GregorianCalendar.SEPTEMBER:
                return 9;
            case GregorianCalendar.OCTOBER:
                return 10;
            case GregorianCalendar.NOVEMBER:
                return 11;
            case GregorianCalendar.DECEMBER:
                return 12;
            default:
                assert false : "Wrong month number: " + month;
        }
        return 0;
    }

    static Locale getLocByID(String locID) throws
            JTZLocaleException {
        Locale loc = null;
        if (locID == null) {
            locID = Locale.getDefault().toString();
        } else if (locID.isEmpty()) {
            locID = Locale.getDefault().toString();
        }
        for (Locale l : Locale.getAvailableLocales()) {
            if ((l.toString().toLowerCase()).equals(locID.toLowerCase())) {
                loc = l;
                break;
            }
        }
        if (loc == null) {
            String msg = "Wrong locale: " + locID;
            throw new JTZLocaleException(msg);
        }
        return loc;
    }

    static TimeZone getTzByID(String timeZoneID) throws
            JTZTimeZoneException {
        TimeZone tz = null;
        boolean foundFl = false;
        if (timeZoneID == null) {
            tz = TimeZone.getTimeZone(GMT_ID);
        } else if (timeZoneID.isEmpty()) {
            tz = TimeZone.getTimeZone(GMT_ID);
        } else {
            String tZArr[] = TimeZone.getAvailableIDs();
            for (String s : tZArr) {
                if (s.toLowerCase().equals(timeZoneID.toLowerCase())) {
                    timeZoneID = s;
                    foundFl = true;
                    break;
                }
            }
            if (foundFl) {
                tz = TimeZone.getTimeZone(timeZoneID);
            } else {
                String msg = "Wrong time zone: " + timeZoneID;
                throw new JTZTimeZoneException(msg);
            }
        }
        return tz;
    }
}
