 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Java Time Zones
 *    Copyright (c) 2011 Marcin Gutman
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.javatimezones;

import java.util.ArrayList;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.TimeZone;
import name.mgplldz.javatimezones.JTZExceptions.JTZFieldException;
import name.mgplldz.javatimezones.JTZExceptions.JTZFormatException;
import name.mgplldz.javatimezones.JTZExceptions.JTZLocaleException;
import name.mgplldz.javatimezones.JTZExceptions.JTZTimeZoneException;

public class JTZUnix {

    /*
     * Public
     */
    public static long unixAdd(long millis, String tzID, int field, int value)
            throws
            JTZTimeZoneException {
        TimeZone tz = JTZHelp.getTzByID(tzID);
        return unixAdd(millis, tz, field, value);
    }

    public static String unixFormat(long millis, String formatStr,
            String tzID, String locID) throws
            JTZTimeZoneException,
            JTZLocaleException,
            JTZFormatException {
        if (formatStr == null) {
            formatStr = "";
        }
        formatStr = formatStr.isEmpty()
                ? JTZHelp.DATE_FRMT_STR_DEFAULT
                : formatStr;
        JTZDateFormat dfMy = new JTZDateFormat(millis, formatStr, tzID, locID);
        return dfMy.getDateStr();
    }

    public static boolean unixIsDST(long millis, String timeZoneID) throws
            JTZTimeZoneException {
        return JTZHelp.getTzByID(timeZoneID).inDaylightTime(new Date(millis));
    }

    public static long unixNow() {
        return System.currentTimeMillis();
    }

    public static long[] unixSplit(long millis, String tzID,
            int... fields)
            throws
            JTZTimeZoneException,
            JTZFieldException {
        TimeZone tz = JTZHelp.getTzByID(tzID);
        return unixSplit(millis, tz, fields);
    }

    public static int unixTzOffset(long millis, String timeZoneID,
            int offsetType)
            throws JTZTimeZoneException,
            JTZFieldException {
        TimeZone tz = JTZHelp.getTzByID(timeZoneID);
        int offset;
        switch (offsetType) {
            case GregorianCalendar.ZONE_OFFSET:
                offset = tz.getRawOffset();
                break;
            case GregorianCalendar.DST_OFFSET:
                offset = tz.getOffset(millis) - tz.getRawOffset();
                break;
            case JTZHelp.TOTAL_OFFSET:
                offset = tz.getOffset(millis);
                break;
            default:
                throw new JTZFieldException("" + offsetType);
        }
        return offset;
    }

    public static String timer(long millis) {
        Integer arrInt[] = timerSplit(millis);
        StringBuffer out = new StringBuffer();
        if (millis < 0) {
            out.append("-");
        }
        for (int i = 0; i < arrInt.length - 1; ++i) {
            out.append(Math.abs(arrInt[i]));
            if (i < arrInt.length - 2) {
                out.append(":");
            }
        }
        out.append(".");
        out.append(Math.abs(arrInt[arrInt.length - 1]));
        return out.toString();
    }

    public static Integer[] timerSplit(long millis) {
        int h = 0;
        int m = 0;
        int s = 0;
        int ss = 0;
        h = (int) (millis / JTZHelp.hourMls());
        millis -= h * JTZHelp.hourMls();
        m = (int) (millis / JTZHelp.minMls());
        millis -= m * JTZHelp.minMls();
        s = (int) (millis / JTZHelp.secMls());
        millis -= s * JTZHelp.secMls();
        ss = (int) (millis);
        ArrayList<Integer> lst = new ArrayList<Integer>();
        lst.add(h);
        lst.add(m);
        lst.add(s);
        lst.add(ss);
        Integer outArr[] = new Integer[lst.size()];
        lst.toArray(outArr);
        lst.clear();
        return outArr;
    }

    /*
     * Package
     */
    static long unixAdd(long millis, TimeZone tz,
            int field, int value)
            throws
            JTZTimeZoneException {
        GregorianCalendar cal = new GregorianCalendar();
        cal.setTimeZone(tz);
        cal.setTimeInMillis(millis);
        cal.add(field, value);
        return cal.getTimeInMillis();
    }

    static long[] unixSplit(long millis, TimeZone tz,
            int... fields)
            throws
            JTZTimeZoneException,
            JTZFieldException {
        long valuesArr[] = null;
        ArrayList<Long> valuesLst = new ArrayList<Long>();
        GregorianCalendar gc = new GregorianCalendar();
        gc.setTimeZone(tz);
        gc.setTimeInMillis(millis);
        if (fields == null) {
            fields = new int[0];
        }
        if (fields.length <= 0) {
            valuesLst.add(new Long(gc.get(JTZHelp.fYear())));
            valuesLst.add(new Long(JTZHelp.monthNb(gc.get(JTZHelp.fMonth()))));
            valuesLst.add(new Long(gc.get(JTZHelp.fDay())));
            valuesLst.add(new Long(gc.get(JTZHelp.fHour())));
            valuesLst.add(new Long(gc.get(JTZHelp.fMinute())));
            valuesLst.add(new Long(gc.get(JTZHelp.fSecond())));
            valuesLst.add(new Long(gc.get(JTZHelp.fMillis())));
            valuesLst.add(new Long(gc.get(JTZHelp.fRawOffset())));
            valuesLst.add(new Long(gc.get(JTZHelp.fDstOffset())));
            valuesLst.add(new Long(gc.get(JTZHelp.fRawOffset())
                    + gc.get(JTZHelp.fDstOffset())));
        } else {
            for (int i : fields) {
                switch (i) {
                    case GregorianCalendar.YEAR:
                        valuesLst.add(new Long(gc.get(JTZHelp.fYear())));
                        break;
                    case GregorianCalendar.MONTH:
                        valuesLst.add(new Long(JTZHelp.monthNb(gc.get(JTZHelp.fMonth()))));
                        break;
                    case GregorianCalendar.DAY_OF_MONTH:
                        valuesLst.add(new Long(gc.get(JTZHelp.fDay())));
                        break;
                    case GregorianCalendar.HOUR_OF_DAY:
                        valuesLst.add(new Long(gc.get(JTZHelp.fHour())));
                        break;
                    case GregorianCalendar.MINUTE:
                        valuesLst.add(new Long(gc.get(JTZHelp.fMinute())));
                        break;
                    case GregorianCalendar.SECOND:
                        valuesLst.add(new Long(gc.get(JTZHelp.fSecond())));
                        break;
                    case GregorianCalendar.MILLISECOND:
                        valuesLst.add(new Long(gc.get(JTZHelp.fMillis())));
                        break;
                    case GregorianCalendar.ZONE_OFFSET:
                        valuesLst.add(new Long(gc.get(JTZHelp.fRawOffset())));
                        break;
                    case GregorianCalendar.DST_OFFSET:
                        valuesLst.add(new Long(gc.get(JTZHelp.fDstOffset())));
                        break;
                    case JTZHelp.TOTAL_OFFSET:
                        valuesLst.add(new Long(gc.get(JTZHelp.fRawOffset()
                                + gc.get(JTZHelp.fDstOffset()))));
                        break;
                    default:
                        throw new JTZFieldException("" + i);
                }
            }
        }
        valuesArr = new long[valuesLst.size()];
        int i = 0;
        for (long l : valuesLst) {
            valuesArr[i] = valuesLst.get(i);
            ++i;
        }
        valuesLst.clear();
        return valuesArr;
    }
}
