 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Java Time Zones
 *    Copyright (c) 2011 Marcin Gutman
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.javatimezones.test;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.TimeZone;
import name.mgplldz.javatimezones.JTZDate;
import name.mgplldz.javatimezones.JTZExceptions.JTZExtractTzException;
import name.mgplldz.javatimezones.JTZExceptions.JTZFormatException;
import name.mgplldz.javatimezones.JTZExceptions.JTZLocaleException;
import name.mgplldz.javatimezones.JTZExceptions.JTZTimeZoneException;
import name.mgplldz.javatimezones.JTZUnix;

public class DateParse {

    public static void main(String args[])
            throws
            JTZTimeZoneException,
            JTZLocaleException,
            JTZFormatException,
            JTZExtractTzException,
            ParseException {
        long millis = 0;
        String date = "2011-10-30 02:59:59:999 cest";
        String format = "yyyy-MM-dd HH:mm:ss:S z";
        System.out.println(JTZDate.dateParse(date, format, ""));
        System.out.println("///////////////////");
        date = "2011-10-30 03:00:00:000 cest";
        format = "yyyy-MM-dd HH:mm:ss:S z";
        millis = JTZDate.dateParse(date, format, "");
        System.out.println(millis);
        SimpleDateFormat sdf = new SimpleDateFormat(format);
        sdf.setCalendar(new GregorianCalendar());
        System.out.println(sdf.parse(date).getTime());
        System.out.println(sdf.format(new Date(millis)));
        System.out.println("///////////////////");
        String tz = "Europe/Warsaw";
        date = "2011-10-30 01:59:59:999";
        format = "yyyy-MM-dd HH:mm:ss:S";
        System.out.println(JTZDate.dateParse(date, format, tz));
        System.out.println("///////////////////");
        tz = "Europe/Warsaw";
        date = "2011-10-30 02:00:00:000";
        format = "yyyy-MM-dd HH:mm:ss:S";
        millis = JTZDate.dateParse(date, format, tz);
        System.out.println(millis);
        System.out.println("///////////////////");
        format = "yyyy-MM-dd HH:mm:ss:S z";
        System.out.println(JTZUnix.unixFormat(millis, format, tz, ""));
        sdf = new SimpleDateFormat(format);
        sdf.setCalendar(new GregorianCalendar());
        sdf.setTimeZone(TimeZone.getTimeZone(tz));
        System.out.println(sdf.format(new Date(millis)));
        System.out.println("///////////////////");
        date = "2011-10-19 cest";
        format = "yyyy-MM-dd z";
        System.out.println(JTZDate.dateParse(date, format, ""));
    }
}

